@@ -1,11 +1,11 @@
-#!/bin/sh
+#!/bin/bash
 
 . /lib/dracut-zfs-lib.sh
 
 _do_zpool_export() {
-	local ret=0
-	local errs=""
-	local final="${1}"
+	ret=0
+	errs=""
+	final="${1}"
 
 	info "ZFS: Exporting ZFS storage pools..."
 	errs=$(export_all -F 2>&1)
