@@ -1,11 +1,11 @@
-#!/bin/bash
+#!/usr/bin/env bash
 
 check() {
 	# We depend on udev-rules being loaded
 	[ "${1}" = "-d" ] && return 0
 
 	# Verify the zfs tool chain
-	for tool in "/sbin/zpool" "/sbin/zfs" "/sbin/mount.zfs" ; do
+	for tool in "/usr/bin/zgenhostid" "/sbin/zpool" "/sbin/zfs" "/sbin/mount.zfs" ; do
 		test -x "$tool" || return 1
 	done
 	# Verify grep exists
@@ -38,6 +38,7 @@ install() {
 	inst_rules /lib/udev/rules.d/60-zvol.rules
 	dracut_install hostid
 	dracut_install grep
+	dracut_install /usr/bin/zgenhostid
 	dracut_install /sbin/zfs
 	dracut_install /sbin/zpool
 	# Workaround for zfsonlinux/zfs#4749 by ensuring libgcc_s.so(.1) is included
@@ -58,6 +59,8 @@ install() {
 	dracut_install /sbin/mount.zfs
 	dracut_install /lib/udev/vdev_id
 	dracut_install awk
+	dracut_install basename
+	dracut_install cut
 	dracut_install head
 	dracut_install /lib/udev/zvol_id
 	inst_hook cmdline 95 "${moddir}/parse-zfs.sh"
@@ -81,20 +84,33 @@ install() {
 	fi
 
 	# Synchronize initramfs and system hostid
-	AA=`hostid | cut -b 1,2`
-	BB=`hostid | cut -b 3,4`
-	CC=`hostid | cut -b 5,6`
-	DD=`hostid | cut -b 7,8`
-	printf "\x${DD}\x${CC}\x${BB}\x${AA}" > "${initdir}/etc/hostid"
+	zgenhostid -o "${initdir}/etc/hostid" "$(hostid)"
 
 	if dracut_module_included "systemd"; then
-		mkdir -p "${initdir}/$systemdsystemunitdir/initrd.target.wants"
+		mkdir -p "${initdir}/$systemdsystemunitdir/zfs-import.target.wants"
 		for _item in scan cache ; do
 			dracut_install /usr/lib/systemd/system/zfs-import-$_item.service
-			if ! [ -L "${initdir}/$systemdsystemunitdir/initrd.target.wants"/zfs-import-$_item.service ]; then
-				ln -s ../zfs-import-$_item.service "${initdir}/$systemdsystemunitdir/initrd.target.wants"/zfs-import-$_item.service
+			if ! [ -L "${initdir}/$systemdsystemunitdir/zfs-import.target.wants"/zfs-import-$_item.service ]; then
+				ln -s ../zfs-import-$_item.service "${initdir}/$systemdsystemunitdir/zfs-import.target.wants"/zfs-import-$_item.service
 				type mark_hostonly >/dev/null 2>&1 && mark_hostonly /usr/lib/systemd/system/zfs-import-$_item.service
 			fi
 		done
+		inst "${moddir}"/zfs-env-bootfs.service "${systemdsystemunitdir}"/zfs-env-bootfs.service
+		ln -s ../zfs-env-bootfs.service "${initdir}/${systemdsystemunitdir}/zfs-import.target.wants"/zfs-env-bootfs.service
+		type mark_hostonly >/dev/null 2>&1 && mark_hostonly /usr/lib/systemd/system/zfs-env-bootfs.service
+		dracut_install systemd-ask-password
+		dracut_install systemd-tty-ask-password-agent
+		mkdir -p "${initdir}/$systemdsystemunitdir/initrd.target.wants"
+		dracut_install /usr/lib/systemd/system/zfs-import.target
+		if ! [ -L "${initdir}/$systemdsystemunitdir/initrd.target.wants"/zfs-import.target ]; then
+			ln -s ../zfs-import.target "${initdir}/$systemdsystemunitdir/initrd.target.wants"/zfs-import.target
+			type mark_hostonly >/dev/null 2>&1 && mark_hostonly /usr/lib/systemd/system/zfs-import.target
+		fi
+		for _service in zfs-snapshot-bootfs.service zfs-rollback-bootfs.service ; do
+			inst "${moddir}"/$_service "${systemdsystemunitdir}"/$_service
+			if ! [ -L "${initdir}/$systemdsystemunitdir/initrd.target.wants"/$_service ]; then
+				ln -s ../$_service "${initdir}/$systemdsystemunitdir/initrd.target.wants"/$_service
+			fi
+		done
 	fi
 }
