@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 
 . /lib/dracut-zfs-lib.sh
 
@@ -13,7 +13,7 @@ esac
 GENERATOR_FILE=/run/systemd/generator/sysroot.mount
 GENERATOR_EXTENSION=/run/systemd/generator/sysroot.mount.d/zfs-enhancement.conf
 
-if [ -e "$GENERATOR_FILE" -a -e "$GENERATOR_EXTENSION" ] ; then
+if [ -e "$GENERATOR_FILE" ] && [ -e "$GENERATOR_EXTENSION" ] ; then
 	# If the ZFS sysroot.mount flag exists, the initial RAM disk configured
 	# it to mount ZFS on root.  In that case, we bail early.  This flag
 	# file gets created by the zfs-generator program upon successful run.
@@ -34,6 +34,7 @@ info "ZFS: No sysroot.mount exists or zfs-generator did not extend it."
 info "ZFS: Mounting root with the traditional mount-zfs.sh instead."
 
 # Delay until all required block devices are present.
+modprobe zfs 2>/dev/null
 udevadm settle
 
 if [ "${root}" = "zfs:AUTO" ] ; then
@@ -57,30 +58,19 @@ ZFS_POOL="${ZFS_DATASET%%/*}"
 
 if import_pool "${ZFS_POOL}" ; then
 	# Load keys if we can or if we need to
-	if [ $(zpool list -H -o feature@encryption $(echo "${ZFS_POOL}" | awk -F\/ '{print $1}')) == 'active' ]; then
+	if [ $(zpool list -H -o feature@encryption $(echo "${ZFS_POOL}" | awk -F\/ '{print $1}')) = 'active' ]; then
 		# if the root dataset has encryption enabled
-		if $(zfs list -H -o encryption "${ZFS_DATASET}" | grep -q -v off); then
-			# figure out where the root dataset has its key, the keylocation should not be none
-			while true; do
-				if [[ $(zfs list -H -o keylocation "${ZFS_DATASET}") == 'none' ]]; then
-					ZFS_DATASET=$(echo -n "${ZFS_DATASET}" | awk 'BEGIN{FS=OFS="/"}{NF--; print}')
-					if [[ "${ZFS_DATASET}" == '' ]]; then
-						rootok=0
-						break
-					fi
-				else
-					rootok=1
-					break
-				fi
-			done
-			[[ "${rootok}" -eq 0 ]]&& return 1
-			# decrypt them
-			TRY_COUNT=5
-			while [ $TRY_COUNT != 0 ]; do
-				zfs load-key "${ZFS_DATASET}"
-				[ $? == 0 ] && break
-				((TRY_COUNT-=1))
-			done
+		ENCRYPTIONROOT="$(zfs get -H -o value encryptionroot "${ZFS_DATASET}")"
+		if ! [ "${ENCRYPTIONROOT}" = "-" ]; then
+			KEYSTATUS="$(zfs get -H -o value keystatus "${ENCRYPTIONROOT}")"
+			# if the key needs to be loaded
+			if [ "$KEYSTATUS" = "unavailable" ]; then
+				# decrypt them
+				ask_for_password \
+					--tries 5 \
+					--prompt "Encrypted ZFS password for ${ENCRYPTIONROOT}: " \
+					--cmd "zfs load-key '${ENCRYPTIONROOT}'"
+			fi
 		fi
 	fi
 	# Let us tell the initrd to run on shutdown.
